# based on https://github.com/dfm/feedfinder2
import logging

import requests

from bs4 import BeautifulSoup
from six.moves.urllib import parse as urlparse



def coerce_url(url):
    url = url.strip()
    if url.startswith("feed://"):
        return "http://{0}".format(url[7:])
    for proto in ["http://", "https://"]:
        if url.startswith(proto):
            return url
    return "http://{0}".format(url)


def get_page(url):
    try:
        r = requests.get(url)
    except Exception as e:
        logging.warn("Error while getting '{0}'".format(url))
        logging.warn("{0}".format(e))
        return None
    return r.text


def is_rss(url):
    text = get_page(url)
    if text is None:
        return False

    data = text.lower()
    if data.count("<html"):
        return False
    return data.count("<rss")+data.count("<rdf")+data.count("<feed")


def is_rss_data(data):
    data = data.lower()
    if data.count("<html"):
        return False
    return data.count("<rss")+data.count("<rdf")+data.count("<feed")


def is_rss_url(url):
    return any(map(url.lower().endswith,
                   [".rss", ".rdf", ".xml", ".atom"]))



def is_rsslike_url(url):
    return any(map(url.lower().count,
                   ["rss", "rdf", "xml", "atom", "feed"]))


def url_feed_prob(url):
    if "comments" in url:
        return -2
    if "georss" in url:
        return -1
    kw = ["atom", "rss", "rdf", ".xml", "feed"]
    for p, t in zip(range(len(kw), 0, -1), kw):
        if t in url:
            return p
    return 0


def sort_urls(feeds):
    return sorted(list(set(feeds)), key=url_feed_prob, reverse=True)


def find_rss_urls(url):
    # Format the URL properly.
    url = coerce_url(url)

    # Download the requested URL.
    text = get_page(url)
    if text is None:
        print("None")
        return []

    # Check if it is already a feed.
    if is_rss_data(text):
        return [url]

    # Get all links which might be RSS URLs.
    tree = BeautifulSoup(text, "html.parser")
    links = []
    for link in tree.find_all("link"):
        if link.get("type") in ["application/rss+xml",
                                "text/xml",
                                "application/atom+xml",
                                "application/x.atom+xml",
                                "application/x-atom+xml"]:
            url = urlparse.urljoin(url, link.get("href", ""))
            links.append(url)

            
    # Check the detected links.
    urls = list(filter(is_rss, links))
    if len(urls):
        return sort_urls(urls)

    # Look for <a> tags.
    local, remote = [], []
    for a in tree.find_all("a"):
        href = a.get("href", None)
        if href is None:
            continue
        if "://" not in href and is_rss_url(href):
            local.append(href)
        if is_rsslike_url(href):
            remote.append(href)

    # Check the local URLs.
    local = [urlparse.urljoin(url, l) for l in local]
    urls += list(filter(is_rss, local))
    logging.info("Found {0} local <a> links to feeds.".format(len(urls)))
    if len(urls):
        return sort_urls(urls)

    # Check the remote URLs.
    remote = [urlparse.urljoin(url, l) for l in remote]
    urls += list(filter(is_rss, remote))
    logging.info("Found {0} remote <a> links to feeds.".format(len(urls)))
    if len(urls):
        return sort_urls(urls)

    # Guessing potential URLs.
    fns = ["atom.xml", "index.atom", "index.rdf", "rss.xml", "index.xml",
           "index.rss", "feeds/latest.rss", "rssfeed.rdf"]
    urls += list(filter(is_rss, [urlparse.urljoin(url, f)
                                         for f in fns]))
    return sort_urls(urls)




"""
newscrawler.crawler.py
~~~~~~~~~~~~~~~~

This module contains the Crawler class, the main logic of our library. It handles the RSS feed extraction,
and aswell the frameworks for information extraction.
"""


import logging

import feedparser

import pandas as pd

from newspaper import Article

from newscrawler.utils import get_page
from newscrawler.utils import find_rss_urls
    
class Crawler:

    NEWSKEYS = ['title',
                'summary',
                'author',
                'published',
                'link',
                'tags',
                'text']

    def __init__(self, url):
        self.url = url
        self.rss_feed = self.get_rss_feed()
        

    def get_rss_feed(self):
        return find_rss_urls(self.url)[0]

    def _extract_article_text_from_html(self, html):
        article = Article('')
        article.set_html(html)
        article.parse()
        return article.text
    
    def _extract_article_information_from_html(self, html):
        article = Article('')
        article.set_html(html)
        article.parse()
        
        article_information = {}
        article_information["text"] = article.text
        article_information["summary"] = article.summary
        article_information["author"] = str(article.authors).strip('[]')
        article_information["published"] = article.publish_date
        article_information["tags"] = article.tags
        article_information["title"] = article.title
        return article_information
     
    def get_article_information_as_dataframe(self):
        article_dict = self._extract_article_information()
        return pd.DataFrame.from_dict(article_dict)
    

    def _extract_article_information(self):
        logging.info("parsing: {}".format(self.rss_feed))
        NewsFeed = feedparser.parse(self.rss_feed)
        feed = NewsFeed.entries

        article_information = {}
        for item in feed:
            for key in self.NEWSKEYS:
                value = item.get(key, '')
                if not value and item.get("link", ""):
                    html = get_page(item["link"])
                    article = self._extract_article_information_from_html(html)
                    value = article[key]
                    
                article_information[key] = article_information.get(key, []) + [value]
        return article_information
